
#include "Config.h"

#include <algorithm>
#include <ctype.h>
#include <assert.h>


ConfigItem Config::cfgi_("");


int ConfigItem::countFields() const {
  char const *p = str_.c_str();
  int cnt = 1;
  while (true) {
    p = strchr(p, ':');
    if (!p) break;
    ++cnt;
    ++p;
  }
  return cnt;
}

ConfigField ConfigItem::field(int ix) const {
  char const *p = str_.c_str();
  int cnt = 0;
  while (true) {
    char const *o = p;
    p = strchr(p, ':');
    if (cnt == ix) {
      if (p) {
        return ConfigField(o, p-o);
      }
      return ConfigField(o, strlen(o));
    }
    if (!p) return ConfigField("", 0);
    ++cnt;
    ++p;
  }
}


void Config::setConfig(char const *base, char const *cfg) {
  std::string s(base, cfg-base);
  std::string::size_type p = s.find('=');
  std::string key;
  std::string value;
  if (p == s.npos) {
    key = s;
    value = "1";          //  if a value is not specified, assume by default it means "1"
  }
  else {
    key.assign(s.begin(), s.begin()+p);
    value.assign(s.begin()+p+1, s.begin()+s.size());
  }
  std::transform(key.begin(), key.end(), key.begin(), tolower);
  config_[key] = ConfigItem(value.c_str());
}

void Config::setText(char const *cfg) {
  char const *base = 0;
  while (*cfg) {
    if (isspace(*cfg)) {
      if (base) {
        setConfig(base, cfg);
        base = 0;
      }
      ++cfg;
      continue;
    }
    if (!base) {
      base = cfg;
    }
    ++cfg;
  }
  if (base) {
    setConfig(base, cfg);
  }
}

ConfigItem const &Config::get(char const *name, char const *dflt) const {
  std::string n(name);
  std::transform(n.begin(), n.end(), n.begin(), tolower);
  ConfigMap::const_iterator ptr = config_.find(n);
  if (ptr == config_.end()) {
    return cfgi_;
  }
  return (*ptr).second;
}

int Config::countKeys() const {
  return (int)config_.size();
}
IIterator<std::string> *Config::keys() const {
  class I : public IIterator<std::string> {
    public:
      I(ConfigMap::const_iterator a, ConfigMap::const_iterator b)
        : ptr_(a), end_(b)
      {
      }
      virtual bool More() {
        return ptr_ != end_;
      }
      virtual std::string Next() {
        assert(ptr_ != end_);
        if (ptr_ == end_) return "";
        std::string const &ret = (*ptr_).first;
        ++ptr_;
        return ret;
      }
      virtual void Dispose() {
        delete this;
      }
      ConfigMap::const_iterator ptr_, end_;
  };
  return new I(config_.begin(), config_.end());
}


class ConfigTester {
  public:
    ConfigTester() {
      testEmpty();
      testValues();
    }
    void testEmpty() {
      Config cfg;
      cfg.setText("");
      assert(cfg.countKeys() == 0);
      cfg.keys()->Dispose();
    }
    void testValues() {
      Config cfg;
      cfg.setText("a=2 B=foo:bar c");
      assert(cfg.countKeys() == 3);
      ConfigItem ci = cfg.get("b");
      assert(ci.countFields() == 2);
      ConfigField cf = cfg.get("b").field(1);
      assert(!strcmp(cf.get<std::string>("").c_str(), "bar"));
      assert(cfg.get("c").get<int>() == 1);
      assert(cfg.get("d").get<int>() == 0);
    }
};

ConfigTester sTestConfig;
