
#if !defined(Config_h)
#define Config_h

#include "Helpers.h"

#include <map>
#include <string>


class ConfigField {
  public:
    ConfigField(char const *base, size_t len)
      : str_(base, len)
    {
    }

    template<typename T>
    T get(T dflt = 0) const;

  protected:
    std::string str_;
};

template<> inline int ConfigField::get<int>(int dflt) const {
  char *ret = 0;
  long l = strtol(str_.c_str(), &ret, 10);
  if (l == 0 && (!ret || ret == str_.c_str())) {
    l = dflt;
  }
  return (int)l;
}

template<> inline std::string ConfigField::get<std::string>(std::string dflt) const {
  return (str_.size() == 0) ? dflt : str_;
}

class ConfigItem {
  public:
    ConfigItem() {}
    ConfigItem(char const *str)
      : str_(str)
    {
    }

    int countFields() const;
    ConfigField field(int ix) const;
    template<typename T> T get(T dflt = 0) const { return field(0).get<T>(dflt); }

  protected:
    std::string str_;
};

class Config {
  public:
    Config() {
    }

    void setConfig(char const *base, char const *cfg);
    void setText(char const *cfg);
    ConfigItem const &get(char const *name, char const *dflt = "") const;
    int countKeys() const;
    IIterator<std::string> *keys() const;

  protected:
    typedef std::map<std::string, ConfigItem> ConfigMap;
    ConfigMap config_;
    static ConfigItem cfgi_;
};

#endif  //  Config_h
